"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.searchByFile = exports.searchByUrl = void 0;
var bytes_1 = __importDefault(require("bytes"));
var form_data_1 = __importDefault(require("form-data"));
var fs_1 = __importDefault(require("fs"));
var jsdom_1 = require("jsdom");
var node_fetch_1 = __importDefault(require("node-fetch"));
function getDocument(htmlString) {
    var document = new jsdom_1.JSDOM(htmlString).window.document;
    return document;
}
function fetchDOM(endpoint) {
    return __awaiter(this, void 0, void 0, function () {
        var _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _a = getDocument;
                    return [4 /*yield*/, node_fetch_1.default(endpoint).then(function (res) { return res.text(); })];
                case 1: return [2 /*return*/, _a.apply(void 0, [_b.sent()])];
            }
        });
    });
}
function getAuthToken() {
    return __awaiter(this, void 0, void 0, function () {
        var document, token;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, fetchDOM('https://ascii2d.net/')];
                case 1:
                    document = _a.sent();
                    token = document.querySelector('meta[name="csrf-token"]').content;
                    return [2 /*return*/, token];
            }
        });
    });
}
function parseExternalSource(externalBox) {
    var _a = Array.from(externalBox.querySelectorAll('a')), titleElement = _a[0], authorElement = _a[1];
    return {
        type: externalBox
            .querySelector('img')
            .alt.toLowerCase(),
        title: titleElement.textContent,
        url: titleElement.href,
        author: {
            name: authorElement.textContent,
            url: authorElement.href,
        },
    };
}
function parseExternalInfo(itemBox) {
    var infoHeader = itemBox.querySelector('.info-header');
    if (!infoHeader)
        return;
    var externalBox = itemBox.querySelector('.external');
    if (!externalBox || !externalBox.textContent)
        return;
    var maybeSource = externalBox.querySelectorAll('img,a,a').length === 3;
    return {
        type: 'external',
        ref: infoHeader.textContent,
        content: maybeSource
            ? parseExternalSource(externalBox)
            : externalBox.textContent.trim(),
    };
}
function parseSource(itemBox) {
    var detailBox = itemBox.querySelector('.detail-box');
    if (!detailBox || detailBox.textContent.trim() === '')
        return;
    var h6 = detailBox.querySelector('h6');
    if (!h6)
        return;
    var anchors = Array.from(h6.querySelectorAll('a'));
    if (anchors[0] && anchors[0].textContent === 'amazon') {
        // amazon
        return {
            type: 'amazon',
            title: h6.childNodes[0].textContent.trim(),
            url: anchors[0].href,
        };
    }
    if (anchors.length < 2)
        return;
    var titleElement = anchors[0], authorElement = anchors[1];
    return {
        type: h6.querySelector('small').textContent.trim(),
        title: titleElement.textContent,
        url: titleElement.href,
        author: {
            name: authorElement.textContent,
            url: authorElement.href,
        },
    };
}
function parseItem(itemBox) {
    var hash = itemBox.querySelector('.hash').textContent;
    var _a = itemBox
        .querySelector('small.text-muted')
        .textContent.split(' '), size = _a[0], fileType = _a[1], fileSizeString = _a[2];
    var thumbnailUrl = 'https://ascii2d.net' +
        itemBox.querySelector('.image-box > img').src;
    var _b = size.split('x').map(function (s) { return parseInt(s); }), width = _b[0], height = _b[1];
    var fileSize = bytes_1.default(fileSizeString);
    var item = {
        hash: hash,
        thumbnailUrl: thumbnailUrl,
        width: width,
        height: height,
        fileType: fileType.toLowerCase(),
        fileSize: fileSize,
    };
    item.externalInfo = parseExternalInfo(itemBox);
    item.source = parseSource(itemBox);
    return item;
}
function parseSearchResult(htmlString) {
    var document = getDocument(htmlString);
    var items = Array.from(document.querySelectorAll('.item-box'))
        .slice(1)
        .map(parseItem);
    return items;
}
function getSearchHash(query) {
    var _a;
    return __awaiter(this, void 0, void 0, function () {
        var searchType, token, formData, response, url, searchHash;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    searchType = query instanceof fs_1.default.ReadStream ? 'file' : 'uri';
                    return [4 /*yield*/, getAuthToken()];
                case 1:
                    token = _b.sent();
                    formData = new form_data_1.default();
                    formData.append('authenticity_token', token);
                    formData.append(searchType, query);
                    return [4 /*yield*/, node_fetch_1.default("https://ascii2d.net/search/" + searchType, {
                            method: 'POST',
                            body: formData,
                            redirect: 'manual',
                        })];
                case 2:
                    response = _b.sent();
                    url = response.headers.get('location');
                    if (!url) {
                        throw new Error("Image size is too large");
                    }
                    searchHash = (_a = url.match(/\/([^/]+)$/)) === null || _a === void 0 ? void 0 : _a[1];
                    if (!searchHash) {
                        throw new Error("Invalid image format");
                    }
                    return [2 /*return*/, searchHash];
            }
        });
    });
}
function searchByUrl(imageUrl, mode) {
    if (mode === void 0) { mode = 'color'; }
    return __awaiter(this, void 0, void 0, function () {
        var hash, url, result, items;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, getSearchHash(imageUrl)];
                case 1:
                    hash = _a.sent();
                    url = "https://ascii2d.net/search/" + mode + "/" + hash;
                    return [4 /*yield*/, node_fetch_1.default(url).then(function (res) { return res.text(); })];
                case 2:
                    result = _a.sent();
                    items = parseSearchResult(result);
                    return [2 /*return*/, { url: url, items: items }];
            }
        });
    });
}
exports.searchByUrl = searchByUrl;
function searchByFile(filePath, mode) {
    if (mode === void 0) { mode = 'color'; }
    return __awaiter(this, void 0, void 0, function () {
        var readStream, hash, url, result, items;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    readStream = fs_1.default.createReadStream(filePath);
                    return [4 /*yield*/, getSearchHash(readStream)];
                case 1:
                    hash = _a.sent();
                    url = "https://ascii2d.net/search/" + mode + "/" + hash;
                    return [4 /*yield*/, node_fetch_1.default(url).then(function (res) { return res.text(); })];
                case 2:
                    result = _a.sent();
                    items = parseSearchResult(result);
                    return [2 /*return*/, { url: url, items: items }];
            }
        });
    });
}
exports.searchByFile = searchByFile;
//# sourceMappingURL=index.js.map